@@ -6,11 +6,16 @@
  * distributed in the NASM archive.
  */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <inttypes.h>
+
 #include "nasm.h"
 #include "nasmlib.h"
+#include "hashtbl.h"
 
 /*
  * A local label is one that begins with exactly one period. Things
@@ -29,9 +34,8 @@
 	((c) == '.' || (c) == '@') :                            \
 	((c) == '.'))
 
-#define LABEL_BLOCK  32         /* no. of labels/block */
+#define LABEL_BLOCK  128	/* no. of labels/block */
 #define LBLK_SIZE    (LABEL_BLOCK*sizeof(union label))
-#define LABEL_HASHES 37         /* no. of hash table entries */
 
 #define END_LIST -3             /* don't clash with NO_SEG! */
 #define END_BLOCK -2
@@ -55,12 +59,14 @@
 
 union label {                   /* actual label structures */
     struct {
-        long segment, offset;
+        int32_t segment;
+	int64_t offset;
         char *label, *special;
         int is_global, is_norm;
     } defn;
     struct {
-        long movingon, dummy;
+        int32_t movingon;
+        int64_t dummy;
         union label *next;
     } admin;
 };
@@ -71,19 +77,20 @@ struct permts {                 /* permanent text storage */
     char data[PERMTS_SIZE];     /* ... the data block itself */
 };
 
-extern int global_offset_changed;       /* defined in nasm.c */
+extern bool global_offset_changed;       /* defined in nasm.c */
 
-static union label *ltab[LABEL_HASHES]; /* using a hash table */
-static union label *lfree[LABEL_HASHES];        /* pointer into the above */
+static struct hash_table *ltab;		/* labels hash table */
+static union label *ldata;		/* all label data blocks */
+static union label *lfree;		/* labels free block */
 static struct permts *perm_head;        /* start of perm. text storage */
 static struct permts *perm_tail;        /* end of perm. text storage */
 
 static void init_block(union label *blk);
-static char *perm_copy(char *string1, char *string2);
+static char *perm_copy(const char *string);
 
 static char *prevlabel;
 
-static int initialised = FALSE;
+static bool initialized = false;
 
 char lprefix[PREFIX_MAX] = { 0 };
 char lpostfix[PREFIX_MAX] = { 0 };
@@ -91,91 +98,85 @@ char lpostfix[PREFIX_MAX] = { 0 };
 /*
  * Internal routine: finds the `union label' corresponding to the
  * given label name. Creates a new one, if it isn't found, and if
- * `create' is TRUE.
+ * `create' is true.
  */
 static union label *find_label(char *label, int create)
 {
-    int hash = 0;
-    char *p, *prev;
-    int prevlen;
-    union label *lptr;
+    char *prev;
+    int prevlen, len;
+    union label *lptr, **lpp;
+    char label_str[IDLEN_MAX];
+    struct hash_insert ip;
 
-    if (islocal(label))
+    if (islocal(label)) {
         prev = prevlabel;
-    else
+	prevlen = strlen(prev);
+	len = strlen(label);
+	if (prevlen+len >= IDLEN_MAX)
+	    return NULL;	/* Error... */
+	memcpy(label_str, prev, prevlen);
+	memcpy(label_str+prevlen, label, len+1);
+	label = label_str;
+    } else {
         prev = "";
-    prevlen = strlen(prev);
-    p = prev;
-    while (*p)
-        hash += *p++;
-    p = label;
-    while (*p)
-        hash += *p++;
-    hash %= LABEL_HASHES;
-    lptr = ltab[hash];
-    while (lptr->admin.movingon != END_LIST) {
-        if (lptr->admin.movingon == END_BLOCK) {
-            lptr = lptr->admin.next;
-            if (!lptr)
-                break;
-        }
-        if (!strncmp(lptr->defn.label, prev, prevlen) &&
-            !strcmp(lptr->defn.label + prevlen, label))
-            return lptr;
-        lptr++;
+	prevlen = 0;
     }
-    if (create) {
-        if (lfree[hash]->admin.movingon == END_BLOCK) {
-            /*
-             * must allocate a new block
-             */
-            lfree[hash]->admin.next =
-                (union label *)nasm_malloc(LBLK_SIZE);
-            lfree[hash] = lfree[hash]->admin.next;
-            init_block(lfree[hash]);
-        }
 
-        lfree[hash]->admin.movingon = BOGUS_VALUE;
-        lfree[hash]->defn.label = perm_copy(prev, label);
-        lfree[hash]->defn.special = NULL;
-        lfree[hash]->defn.is_global = NOT_DEFINED_YET;
-        return lfree[hash]++;
-    } else
-        return NULL;
+    lpp = (union label **) hash_find(ltab, label, &ip);
+    lptr = lpp ? *lpp : NULL;
+
+    if (lptr || !create)
+	return lptr;
+
+    /* Create a new label... */
+    if (lfree->admin.movingon == END_BLOCK) {
+	/*
+	 * must allocate a new block
+	 */
+	lfree->admin.next =
+	    (union label *)nasm_malloc(LBLK_SIZE);
+	lfree = lfree->admin.next;
+	init_block(lfree);
+    }
+
+    lfree->admin.movingon = BOGUS_VALUE;
+    lfree->defn.label = perm_copy(label);
+    lfree->defn.special = NULL;
+    lfree->defn.is_global = NOT_DEFINED_YET;
+
+    hash_add(&ip, lfree->defn.label, lfree);
+    return lfree++;
 }
 
-int lookup_label(char *label, long *segment, long *offset)
+bool lookup_label(char *label, int32_t *segment, int64_t *offset)
 {
     union label *lptr;
 
-    if (!initialised)
-        return 0;
+    if (!initialized)
+        return false;
 
     lptr = find_label(label, 0);
     if (lptr && (lptr->defn.is_global & DEFINED_BIT)) {
         *segment = lptr->defn.segment;
         *offset = lptr->defn.offset;
-        return 1;
+        return true;
     } else
-        return 0;
+        return false;
 }
 
-int is_extern(char *label)
+bool is_extern(char *label)
 {
     union label *lptr;
 
-    if (!initialised)
-        return 0;
+    if (!initialized)
+        return false;
 
     lptr = find_label(label, 0);
-    if (lptr && (lptr->defn.is_global & EXTERN_BIT))
-        return 1;
-    else
-        return 0;
+    return (lptr && (lptr->defn.is_global & EXTERN_BIT));
 }
 
-void redefine_label(char *label, long segment, long offset, char *special,
-                    int is_norm, int isextrn, struct ofmt *ofmt,
+void redefine_label(char *label, int32_t segment, int64_t offset, char *special,
+                    bool is_norm, bool isextrn, struct ofmt *ofmt,
                     efunc error)
 {
     union label *lptr;
@@ -247,8 +248,8 @@ void redefine_label(char *label, long segment, long offset, char *special,
     /* if (pass0 == 1) */
 }
 
-void define_label(char *label, long segment, long offset, char *special,
-                  int is_norm, int isextrn, struct ofmt *ofmt, efunc error)
+void define_label(char *label, int32_t segment, int64_t offset, char *special,
+                  bool is_norm, bool isextrn, struct ofmt *ofmt, efunc error)
 {
     union label *lptr;
     int exi;
@@ -314,7 +315,7 @@ void define_label(char *label, long segment, long offset, char *special,
     }                           /* if (pass0 == 1) */
 }
 
-void define_common(char *label, long segment, long size, char *special,
+void define_common(char *label, int32_t segment, int32_t size, char *special,
                    struct ofmt *ofmt, efunc error)
 {
     union label *lptr;
@@ -355,7 +356,7 @@ void declare_as_global(char *label, char *special, efunc error)
     switch (lptr->defn.is_global & TYPE_MASK) {
     case NOT_DEFINED_YET:
         lptr->defn.is_global = GLOBAL_PLACEHOLDER;
-        lptr->defn.special = special ? perm_copy(special, "") : NULL;
+        lptr->defn.special = special ? perm_copy(special) : NULL;
         break;
     case GLOBAL_PLACEHOLDER:   /* already done: silently ignore */
     case GLOBAL_SYMBOL:
@@ -370,51 +371,39 @@ void declare_as_global(char *label, char *special, efunc error)
 
 int init_labels(void)
 {
-    int i;
+    ltab = hash_init();
 
-    for (i = 0; i < LABEL_HASHES; i++) {
-        ltab[i] = (union label *)nasm_malloc(LBLK_SIZE);
-        if (!ltab[i])
-            return -1;          /* can't initialise, panic */
-        init_block(ltab[i]);
-        lfree[i] = ltab[i];
-    }
+    ldata = lfree = (union label *)nasm_malloc(LBLK_SIZE);
+    init_block(lfree);
 
     perm_head =
         perm_tail = (struct permts *)nasm_malloc(sizeof(struct permts));
 
-    if (!perm_head)
-        return -1;
-
     perm_head->next = NULL;
     perm_head->size = PERMTS_SIZE;
     perm_head->usage = 0;
 
     prevlabel = "";
 
-    initialised = TRUE;
+    initialized = true;
 
     return 0;
 }
 
 void cleanup_labels(void)
 {
-    int i;
+    union label *lptr, *lhold;
 
-    initialised = FALSE;
+    initialized = false;
 
-    for (i = 0; i < LABEL_HASHES; i++) {
-        union label *lptr, *lhold;
+    hash_free(ltab);
 
-        lptr = lhold = ltab[i];
-
-        while (lptr) {
-            while (lptr->admin.movingon != END_BLOCK)
-                lptr++;
-            lptr = lptr->admin.next;
-            nasm_free(lhold);
-            lhold = lptr;
-        }
+    lptr = lhold = ldata;
+    while (lptr) {
+	lptr = &lptr[LABEL_BLOCK-1];
+	lptr = lptr->admin.next;
+	nasm_free(lhold);
+	lhold = lptr;
     }
 
     while (perm_head) {
@@ -434,10 +423,10 @@ static void init_block(union label *blk)
     blk[LABEL_BLOCK - 1].admin.next = NULL;
 }
 
-static char *perm_copy(char *string1, char *string2)
+static char *perm_copy(const char *string)
 {
-    char *p, *q;
-    int len = strlen(string1) + strlen(string2) + 1;
+    char *p;
+    int len = strlen(string)+1;
 
     if (perm_tail->size - perm_tail->usage < len) {
         perm_tail->next =
@@ -447,11 +436,9 @@ static char *perm_copy(char *string1, char *string2)
         perm_tail->size = PERMTS_SIZE;
         perm_tail->usage = 0;
     }
-    p = q = perm_tail->data + perm_tail->usage;
-    while ((*q = *string1++))
-        q++;
-    while ((*q++ = *string2++)) ;
-    perm_tail->usage = q - perm_tail->data;
+    p = perm_tail->data + perm_tail->usage;
+    memcpy(p, string, len);
+    perm_tail->usage += len;
 
     return p;
 }
