@@ -6,19 +6,24 @@
  * distributed in the NASM archive.
  */
 
+#ifndef LABELS_H
+#define LABELS_H
+
 extern char lprefix[PREFIX_MAX];
 extern char lpostfix[PREFIX_MAX];
 
-int lookup_label(char *label, long *segment, long *offset);
-int is_extern(char *label);
-void define_label(char *label, long segment, long offset, char *special,
-                  int is_norm, int isextrn, struct ofmt *ofmt,
+bool lookup_label(char *label, int32_t *segment, int64_t *offset);
+bool is_extern(char *label);
+void define_label(char *label, int32_t segment, int64_t offset, char *special,
+		  bool is_norm, bool isextrn, struct ofmt *ofmt,
                   efunc error);
-void redefine_label(char *label, long segment, long offset, char *special,
-                    int is_norm, int isextrn, struct ofmt *ofmt,
+void redefine_label(char *label, int32_t segment, int64_t offset, char *special,
+                    bool is_norm, bool isextrn, struct ofmt *ofmt,
                     efunc error);
-void define_common(char *label, long segment, long size, char *special,
+void define_common(char *label, int32_t segment, int32_t size, char *special,
                    struct ofmt *ofmt, efunc error);
 void declare_as_global(char *label, char *special, efunc error);
 int init_labels(void);
 void cleanup_labels(void);
+
+#endif /* LABELS_H */
