@@ -7,13 +7,17 @@
  * distributed in the NASM archive.
  */
 
+#include "compiler.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "stdscan.h"
 #include "outform.h"
 
 #ifdef OF_OBJ
@@ -131,8 +135,8 @@ struct ObjRecord {
     ObjRecord *child;           /* Associated record below this one */
     ObjRecord **up;             /* Master pointer to this ObjRecord */
     ObjRecord *back;            /* Previous part of this record     */
-    unsigned long parm[OBJ_PARMS];      /* Parameters for ori routine       */
-    unsigned char buf[RECORD_MAX + 3];
+    uint32_t parm[OBJ_PARMS];      /* Parameters for ori routine       */
+    uint8_t buf[RECORD_MAX + 3];
 };
 
 static void obj_fwrite(ObjRecord * orp);
@@ -141,8 +145,8 @@ static void ori_pubdef(ObjRecord * orp);
 static void ori_null(ObjRecord * orp);
 static ObjRecord *obj_commit(ObjRecord * orp);
 
-static int obj_uppercase;       /* Flag: all names in uppercase */
-static int obj_use32;           /* Flag: at least one segment is 32-bit */
+static bool obj_uppercase;       /* Flag: all names in uppercase */
+static bool obj_use32;           /* Flag: at least one segment is 32-bit */
 
 /*
  * Clear an ObjRecord structure.  (Never reallocates).
@@ -268,7 +272,7 @@ static ObjRecord *obj_commit(ObjRecord * orp)
 /*
  * Write a byte
  */
-static ObjRecord *obj_byte(ObjRecord * orp, unsigned char val)
+static ObjRecord *obj_byte(ObjRecord * orp, uint8_t val)
 {
     orp = obj_check(orp, 1);
     orp->buf[orp->used] = val;
@@ -303,7 +307,7 @@ static ObjRecord *obj_rword(ObjRecord * orp, unsigned int val)
 /*
  * Write a dword
  */
-static ObjRecord *obj_dword(ObjRecord * orp, unsigned long val)
+static ObjRecord *obj_dword(ObjRecord * orp, uint32_t val)
 {
     orp = obj_check(orp, 4);
     orp->buf[orp->used] = val;
@@ -336,7 +340,7 @@ static ObjRecord *obj_force(ObjRecord * orp, int x)
  * This routine writes a field of size x.  The caller does not need to worry at
  * all about whether 16-bits or 32-bits are required.
  */
-static ObjRecord *obj_x(ObjRecord * orp, unsigned long val)
+static ObjRecord *obj_x(ObjRecord * orp, uint32_t val)
 {
     if (orp->type & 1)
         orp->x_size = 32;
@@ -361,7 +365,7 @@ static ObjRecord *obj_index(ObjRecord * orp, unsigned int val)
 /*
  * Writes a variable length value
  */
-static ObjRecord *obj_value(ObjRecord * orp, unsigned long val)
+static ObjRecord *obj_value(ObjRecord * orp, uint32_t val)
 {
     if (val <= 128)
         return (obj_byte(orp, val));
@@ -381,7 +385,7 @@ static ObjRecord *obj_value(ObjRecord * orp, unsigned long val)
 static ObjRecord *obj_name(ObjRecord * orp, char *name)
 {
     int len = strlen(name);
-    unsigned char *ptr;
+    uint8_t *ptr;
 
     orp = obj_check(orp, len + 1);
     ptr = orp->buf + orp->used;
@@ -464,8 +468,8 @@ static efunc error;
 static evalfunc evaluate;
 static ldfunc deflabel;
 static FILE *ofp;
-static long first_seg;
-static int any_segs;
+static int32_t first_seg;
+static bool any_segs;
 static int passtwo;
 static int arrindex;
 
@@ -479,8 +483,8 @@ struct Group;
 struct LineNumber {
     struct LineNumber *next;
     struct Segment *segment;
-    long offset;
-    long lineno;
+    int32_t offset;
+    int32_t lineno;
 };
 
 static struct FileName {
@@ -501,16 +505,16 @@ static struct Array {
 static struct Public {
     struct Public *next;
     char *name;
-    long offset;
-    long segment;               /* only if it's far-absolute */
+    int32_t offset;
+    int32_t segment;               /* only if it's far-absolute */
     int type;                   /* only for local debug syms */
 } *fpubhead, **fpubtail, *last_defined;
 
 static struct External {
     struct External *next;
     char *name;
-    long commonsize;
-    long commonelem;            /* element size if FAR, else zero */
+    int32_t commonsize;
+    int32_t commonelem;            /* element size if FAR, else zero */
     int index;                  /* OBJ-file external index */
     enum {
         DEFWRT_NONE,            /* no unusual default-WRT */
@@ -535,18 +539,18 @@ static struct ExtBack {
 
 static struct Segment {
     struct Segment *next;
-    long index;                 /* the NASM segment id */
-    long obj_index;             /* the OBJ-file segment index */
-    struct Group *grp;          /* the group it belongs to */
-    unsigned long currentpos;
-    long align;                 /* can be SEG_ABS + absolute addr */
+    int32_t index;                 /* the NASM segment id */
+    int32_t obj_index;             /* the OBJ-file segment index */
+    struct Group *grp;          /* the group it beint32_ts to */
+    uint32_t currentpos;
+    int32_t align;                 /* can be SEG_ABS + absolute addr */
     enum {
         CMB_PRIVATE = 0,
         CMB_PUBLIC = 2,
         CMB_STACK = 5,
         CMB_COMMON = 6
     } combine;
-    long use32;                 /* is this segment 32-bit? */
+    bool use32;                 /* is this segment 32-bit? */
     struct Public *pubhead, **pubtail, *lochead, **loctail;
     char *name;
     char *segclass, *overlay;   /* `class' is a C++ keyword :-) */
@@ -556,12 +560,12 @@ static struct Segment {
 static struct Group {
     struct Group *next;
     char *name;
-    long index;                 /* NASM segment id */
-    long obj_index;             /* OBJ-file group index */
-    long nentries;              /* number of elements... */
-    long nindices;              /* ...and number of index elts... */
+    int32_t index;                 /* NASM segment id */
+    int32_t obj_index;             /* OBJ-file group index */
+    int32_t nentries;              /* number of elements... */
+    int32_t nindices;              /* ...and number of index elts... */
     union {
-        long index;
+        int32_t index;
         char *name;
     } segs[GROUP_MAX];          /* ...in this */
 } *grphead, **grptail, *obj_grp_needs_update;
@@ -587,14 +591,14 @@ static struct ExpDef {
 #define EXPDEF_FLAG_NODATA   0x20
 #define EXPDEF_MASK_PARMCNT  0x1F
 
-static long obj_entry_seg, obj_entry_ofs;
+static int32_t obj_entry_seg, obj_entry_ofs;
 
 struct ofmt of_obj;
 
 /* The current segment */
 static struct Segment *current_seg;
 
-static long obj_segment(char *, int, int *);
+static int32_t obj_segment(char *, int, int *);
 static void obj_write_file(int debuginfo);
 static int obj_directive(char *, char *, int);
 
@@ -605,7 +609,7 @@ static void obj_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
     evaluate = eval;
     deflabel = ldef;
     first_seg = seg_alloc();
-    any_segs = FALSE;
+    any_segs = false;
     fpubhead = NULL;
     fpubtail = &fpubhead;
     exthead = NULL;
@@ -623,8 +627,8 @@ static void obj_init(FILE * fp, efunc errfunc, ldfunc ldef, evalfunc eval)
     grphead = obj_grp_needs_update = NULL;
     grptail = &grphead;
     obj_entry_seg = NO_SEG;
-    obj_uppercase = FALSE;
-    obj_use32 = FALSE;
+    obj_uppercase = false;
+    obj_use32 = false;
     passtwo = 0;
     current_seg = NULL;
 
@@ -721,8 +725,8 @@ static void obj_ext_set_defwrt(struct External *ext, char *id)
     dws = ext;
 }
 
-static void obj_deflabel(char *name, long segment,
-                         long offset, int is_global, char *special)
+static void obj_deflabel(char *name, int32_t segment,
+                         int64_t offset, int is_global, char *special)
 {
     /*
      * We have three cases:
@@ -744,7 +748,7 @@ static void obj_deflabel(char *name, long segment,
     struct ExtBack *eb;
     struct Segment *seg;
     int i;
-    int used_special = FALSE;   /* have we used the special text? */
+    bool used_special = false;   /* have we used the special text? */
 
 #if defined(DEBUG) && DEBUG>2
     fprintf(stderr,
@@ -805,7 +809,7 @@ static void obj_deflabel(char *name, long segment,
     }
 
     /*
-     * If `any_segs' is still FALSE, we might need to define a
+     * If `any_segs' is still false, we might need to define a
      * default segment, if they're trying to declare a label in
      * `first_seg'.
      */
@@ -881,7 +885,7 @@ static void obj_deflabel(char *name, long segment,
      * specifications.
      */
     while (special && *special) {
-        used_special = TRUE;
+        used_special = true;
 
         /*
          * We might have a default-WRT specification.
@@ -989,15 +993,15 @@ static void obj_deflabel(char *name, long segment,
 
 /* forward declaration */
 static void obj_write_fixup(ObjRecord * orp, int bytes,
-                            int segrel, long seg, long wrt,
+                            int segrel, int32_t seg, int32_t wrt,
                             struct Segment *segto);
 
-static void obj_out(long segto, const void *data, unsigned long type,
-                    long segment, long wrt)
+static void obj_out(int32_t segto, const void *data,
+		    enum out_type type, uint64_t size,
+                    int32_t segment, int32_t wrt)
 {
-    unsigned long size, realtype;
-    const unsigned char *ucdata;
-    long ldata;
+    const uint8_t *ucdata;
+    int32_t ldata;
     struct Segment *seg;
     ObjRecord *orp;
 
@@ -1005,14 +1009,14 @@ static void obj_out(long segto, const void *data, unsigned long type,
      * handle absolute-assembly (structure definitions)
      */
     if (segto == NO_SEG) {
-        if ((type & OUT_TYPMASK) != OUT_RESERVE)
+        if (type != OUT_RESERVE)
             error(ERR_NONFATAL, "attempt to assemble code in [ABSOLUTE]"
                   " space");
         return;
     }
 
     /*
-     * If `any_segs' is still FALSE, we must define a default
+     * If `any_segs' is still false, we must define a default
      * segment.
      */
     if (!any_segs) {
@@ -1033,9 +1037,7 @@ static void obj_out(long segto, const void *data, unsigned long type,
     orp = seg->orp;
     orp->parm[0] = seg->currentpos;
 
-    size = type & OUT_SIZMASK;
-    realtype = type & OUT_TYPMASK;
-    if (realtype == OUT_RAWDATA) {
+    if (type == OUT_RAWDATA) {
         ucdata = data;
         while (size > 0) {
             unsigned int len;
@@ -1049,22 +1051,22 @@ static void obj_out(long segto, const void *data, unsigned long type,
             ucdata += len;
             size -= len;
         }
-    } else if (realtype == OUT_ADDRESS || realtype == OUT_REL2ADR ||
-               realtype == OUT_REL4ADR) {
+    } else if (type == OUT_ADDRESS || type == OUT_REL2ADR ||
+               type == OUT_REL4ADR) {
         int rsize;
 
-        if (segment == NO_SEG && realtype != OUT_ADDRESS)
+        if (segment == NO_SEG && type != OUT_ADDRESS)
             error(ERR_NONFATAL, "relative call to absolute address not"
                   " supported by OBJ format");
         if (segment >= SEG_ABS)
             error(ERR_NONFATAL, "far-absolute relocations not supported"
                   " by OBJ format");
-        ldata = *(long *)data;
-        if (realtype == OUT_REL2ADR) {
+        ldata = *(int64_t *)data;
+        if (type == OUT_REL2ADR) {
             ldata += (size - 2);
             size = 2;
         }
-        if (realtype == OUT_REL4ADR) {
+        if (type == OUT_REL4ADR) {
             ldata += (size - 4);
             size = 4;
         }
@@ -1089,10 +1091,10 @@ static void obj_out(long segto, const void *data, unsigned long type,
         }
         if (segment != NO_SEG)
             obj_write_fixup(orp, rsize,
-                            (realtype == OUT_ADDRESS ? 0x4000 : 0),
+                            (type == OUT_ADDRESS ? 0x4000 : 0),
                             segment, wrt, seg);
         seg->currentpos += size;
-    } else if (realtype == OUT_RESERVE) {
+    } else if (type == OUT_RESERVE) {
         if (orp->committed)
             orp = obj_bump(orp);
         seg->currentpos += size;
@@ -1101,13 +1103,13 @@ static void obj_out(long segto, const void *data, unsigned long type,
 }
 
 static void obj_write_fixup(ObjRecord * orp, int bytes,
-                            int segrel, long seg, long wrt,
+                            int segrel, int32_t seg, int32_t wrt,
                             struct Segment *segto)
 {
     unsigned locat;
     int method;
     int base;
-    long tidx, fidx;
+    int32_t tidx, fidx;
     struct Segment *s = NULL;
     struct Group *g = NULL;
     struct External *e = NULL;
@@ -1132,14 +1134,14 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
     }
 
     if (seg % 2) {
-        base = TRUE;
+        base = true;
         locat = FIX_16_SELECTOR;
         seg--;
         if (bytes != 2)
             error(ERR_PANIC, "OBJ: 4-byte segment base fixup got"
                   " through sanity check");
     } else {
-        base = FALSE;
+        base = false;
         locat = (bytes == 2) ? FIX_16_OFFSET : FIX_32_OFFSET;
         if (!segrel)
             /*
@@ -1170,9 +1172,9 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
         if (g)
             method = 5, tidx = g->obj_index;
         else {
-            long i = seg / 2;
+            int32_t i = seg / 2;
             struct ExtBack *eb = ebhead;
-            while (i > EXT_BLKSIZ) {
+            while (i >= EXT_BLKSIZ) {
                 if (eb)
                     eb = eb->next;
                 else
@@ -1229,9 +1231,9 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
             if (g)
                 method |= 0x10, fidx = g->obj_index;
             else {
-                long i = wrt / 2;
+                int32_t i = wrt / 2;
                 struct ExtBack *eb = ebhead;
-                while (i > EXT_BLKSIZ) {
+                while (i >= EXT_BLKSIZ) {
                     if (eb)
                         eb = eb->next;
                     else
@@ -1254,7 +1256,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
     obj_commit(forp);
 }
 
-static long obj_segment(char *name, int pass, int *bits)
+static int32_t obj_segment(char *name, int pass, int *bits)
 {
     /*
      * We call the label manager here to define a name for the new
@@ -1275,7 +1277,8 @@ static long obj_segment(char *name, int pass, int *bits)
         struct Segment *seg;
         struct Group *grp;
         struct External **extp;
-        int obj_idx, i, attrs, rn_error;
+        int obj_idx, i, attrs;
+	bool rn_error;
         char *p;
 
         /*
@@ -1326,11 +1329,11 @@ static long obj_segment(char *name, int pass, int *bits)
         seg->index = (any_segs ? seg_alloc() : first_seg);
         seg->obj_index = obj_idx;
         seg->grp = NULL;
-        any_segs = TRUE;
+        any_segs = true;
         seg->name = NULL;
         seg->currentpos = 0;
         seg->align = 1;         /* default */
-        seg->use32 = FALSE;     /* default */
+        seg->use32 = false;     /* default */
         seg->combine = CMB_PUBLIC;      /* default */
         seg->segclass = seg->overlay = NULL;
         seg->pubhead = NULL;
@@ -1364,16 +1367,16 @@ static long obj_segment(char *name, int pass, int *bits)
             else if (!nasm_stricmp(p, "stack"))
                 seg->combine = CMB_STACK;
             else if (!nasm_stricmp(p, "use16"))
-                seg->use32 = FALSE;
+                seg->use32 = false;
             else if (!nasm_stricmp(p, "use32"))
-                seg->use32 = TRUE;
+                seg->use32 = true;
             else if (!nasm_stricmp(p, "flat")) {
                 /*
                  * This segment is an OS/2 FLAT segment. That means
                  * that its default group is group FLAT, even if
                  * the group FLAT does not explicitly _contain_ the
                  * segment.
-                 * 
+                 *
                  * When we see this, we must create the group
                  * `FLAT', containing no segments, if it does not
                  * already exist; then we must set the default
@@ -1453,10 +1456,10 @@ static long obj_segment(char *name, int pass, int *bits)
         obj_seg_needs_update = seg;
         if (seg->align >= SEG_ABS)
             deflabel(name, NO_SEG, seg->align - SEG_ABS,
-                     NULL, FALSE, FALSE, &of_obj, error);
+                     NULL, false, false, &of_obj, error);
         else
             deflabel(name, seg->index + 1, 0L,
-                     NULL, FALSE, FALSE, &of_obj, error);
+                     NULL, false, false, &of_obj, error);
         obj_seg_needs_update = NULL;
 
         /*
@@ -1557,7 +1560,7 @@ static int obj_directive(char *directive, char *value, int pass)
 
             obj_grp_needs_update = grp;
             deflabel(v, grp->index + 1, 0L,
-                     NULL, FALSE, FALSE, &of_obj, error);
+                     NULL, false, false, &of_obj, error);
             obj_grp_needs_update = NULL;
 
             while (*q) {
@@ -1618,7 +1621,7 @@ static int obj_directive(char *directive, char *value, int pass)
         return 1;
     }
     if (!strcmp(directive, "uppercase")) {
-        obj_uppercase = TRUE;
+        obj_uppercase = true;
         return 1;
     }
     if (!strcmp(directive, "import")) {
@@ -1651,7 +1654,7 @@ static int obj_directive(char *directive, char *value, int pass)
                   " and library name");
         else {
             struct ImpDef *imp;
-            int err = FALSE;
+            bool err = false;
 
             imp = *imptail = nasm_malloc(sizeof(struct ImpDef));
             imptail = &imp->next;
@@ -1715,7 +1718,7 @@ static int obj_directive(char *directive, char *value, int pass)
             else if (!nasm_stricmp(v, "nodata"))
                 flags |= EXPDEF_FLAG_NODATA;
             else if (!nasm_strnicmp(v, "parm=", 5)) {
-                int err = FALSE;
+                bool err = false;
                 flags |= EXPDEF_MASK_PARMCNT & readnum(v + 5, &err);
                 if (err) {
                     error(ERR_NONFATAL,
@@ -1723,7 +1726,7 @@ static int obj_directive(char *directive, char *value, int pass)
                     return 1;
                 }
             } else {
-                int err = FALSE;
+                bool err = false;
                 ordinal = readnum(v, &err);
                 if (err) {
                     error(ERR_NONFATAL,
@@ -1747,7 +1750,7 @@ static int obj_directive(char *directive, char *value, int pass)
     return 0;
 }
 
-static long obj_segbase(long segment)
+static int32_t obj_segbase(int32_t segment)
 {
     struct Segment *seg;
 
@@ -1762,11 +1765,11 @@ static long obj_segbase(long segment)
         /*
          * Might be an external with a default WRT.
          */
-        long i = segment / 2;
+        int32_t i = segment / 2;
         struct ExtBack *eb = ebhead;
         struct External *e;
 
-        while (i > EXT_BLKSIZ) {
+        while (i >= EXT_BLKSIZ) {
             if (eb)
                 eb = eb->next;
             else
@@ -1876,7 +1879,7 @@ static void obj_write_file(int debuginfo)
 
     /*
      * Write the first LNAMES record, containing LNAME one, which
-     * is null. Also initialise the LNAME counter.
+     * is null. Also initialize the LNAME counter.
      */
     orp->type = LNAMES;
     obj_byte(orp, 0);
@@ -1907,7 +1910,7 @@ static void obj_write_file(int debuginfo)
     orp->type = SEGDEF;
     for (seg = seghead; seg; seg = seg->next) {
         int acbp;
-        unsigned long seglen = seg->currentpos;
+        uint32_t seglen = seg->currentpos;
 
         acbp = (seg->combine << 2);     /* C field */
 
@@ -1992,7 +1995,7 @@ static void obj_write_file(int debuginfo)
     orp->parm[0] = 0;
     orp->parm[1] = 0;
     for (pub = fpubhead; pub; pub = pub->next) {        /* pub-crawl :-) */
-        if (orp->parm[2] != pub->segment) {
+        if (orp->parm[2] != (uint32_t)pub->segment) {
             obj_emit(orp);
             orp->parm[2] = pub->segment;
         }
@@ -2271,10 +2274,10 @@ static void obj_write_file(int debuginfo)
     nasm_free(orp);
 }
 
-void obj_fwrite(ObjRecord * orp)
+static void obj_fwrite(ObjRecord * orp)
 {
     unsigned int cksum, len;
-    unsigned char *ptr;
+    uint8_t *ptr;
 
     cksum = orp->type;
     if (orp->x_size == 32)
@@ -2282,7 +2285,7 @@ void obj_fwrite(ObjRecord * orp)
     fputc(cksum, ofp);
     len = orp->committed + 1;
     cksum += (len & 0xFF) + ((len >> 8) & 0xFF);
-    fwriteshort(len, ofp);
+    fwriteint16_t(len, ofp);
     fwrite(orp->buf, 1, len - 1, ofp);
     for (ptr = orp->buf; --len; ptr++)
         cksum += *ptr;
@@ -2350,7 +2353,7 @@ static void dbgbi_cleanup(void)
     }
 }
 
-static void dbgbi_linnum(const char *lnfname, long lineno, long segto)
+static void dbgbi_linnum(const char *lnfname, int32_t lineno, int32_t segto)
 {
     struct FileName *fn;
     struct LineNumber *ln;
@@ -2360,7 +2363,7 @@ static void dbgbi_linnum(const char *lnfname, long lineno, long segto)
         return;
 
     /*
-     * If `any_segs' is still FALSE, we must define a default
+     * If `any_segs' is still false, we must define a default
      * segment.
      */
     if (!any_segs) {
@@ -2401,8 +2404,8 @@ static void dbgbi_linnum(const char *lnfname, long lineno, long segto)
     fn->lntail = &ln->next;
 
 }
-static void dbgbi_deflabel(char *name, long segment,
-                           long offset, int is_global, char *special)
+static void dbgbi_deflabel(char *name, int32_t segment,
+                           int64_t offset, int is_global, char *special)
 {
     struct Segment *seg;
 
@@ -2438,7 +2441,7 @@ static void dbgbi_deflabel(char *name, long segment,
     }
 
     /*
-     * If `any_segs' is still FALSE, we might need to define a
+     * If `any_segs' is still false, we might need to define a
      * default segment, if they're trying to declare a label in
      * `first_seg'.  But the label should exist due to a prior
      * call to obj_deflabel so we can skip that.
@@ -2457,7 +2460,7 @@ static void dbgbi_deflabel(char *name, long segment,
             loc->offset = offset;
         }
 }
-static void dbgbi_typevalue(long type)
+static void dbgbi_typevalue(int32_t type)
 {
     int vsize;
     int elem = TYM_ELEMENTS(type);
@@ -2468,7 +2471,7 @@ static void dbgbi_typevalue(long type)
 
     switch (type) {
     case TY_BYTE:
-        last_defined->type = 8; /* unsigned char */
+        last_defined->type = 8; /* uint8_t */
         vsize = 1;
         break;
     case TY_WORD:
